@@ -15,7 +15,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program (see the file COPYING); if not, write to the
- * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Free Software Foundation, Inc.,
+ * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
  ****************************************************************
  * $Id: os.h,v 1.10 1994/05/31 12:32:22 mlschroe Exp $ FAU
@@ -26,7 +27,7 @@
 #include <sys/param.h>
 
 
-#if defined(BSDI) || defined(__386BSD__) || defined(_CX_UX)
+#if defined(BSDI) || defined(__386BSD__) || defined(_CX_UX) || defined(hpux)
 # include <signal.h>
 #endif /* BSDI || __386BSD__ || _CX_UX */
 
@@ -114,11 +115,11 @@ extern int errno;
 #endif
 
 #ifndef USEBCOPY
-# ifdef USEMEMCPY
-#  define bcopy(s,d,len) memcpy(d,s,len)
+# ifdef USEMEMMOVE
+#  define bcopy(s,d,len) memmove(d,s,len)
 # else
-#  ifdef USEMEMMOVE
-#   define bcopy(s,d,len) memmove(d,s,len)
+#  ifdef USEMEMCPY
+#   define bcopy(s,d,len) memcpy(d,s,len)
 #  else
 #   define NEED_OWN_BCOPY
 #   define bcopy xbcopy
@@ -182,15 +183,14 @@ extern int errno;
  * TIOCPKT mode causes data loss if our buffer is too small (IOSIZE)
  * to hold the whole packet at first read().
  * (Marc Boucher)
- */
-#ifdef sgi 
-# undef TIOCPKT
-#endif
-
-/* matthew green:
+ *
+ * matthew green:
  * TIOCPKT is broken on dgux 5.4.1 generic AViiON mc88100
+ *
+ * Joe Traister: On AIX4, programs like irc won't work if screen
+ * uses TIOCPKT (select fails to return on pty read).
  */
-#ifdef DGUX
+#if defined(sgi) || defined(DGUX) || defined(_IBMR2)
 # undef TIOCPKT
 #endif
 
@@ -295,16 +295,31 @@ extern int errno;
 #endif
 
 #if defined(S_IFIFO) && defined(S_IFMT) && !defined(S_ISFIFO)
-#define S_ISFIFO(mode) ((mode & S_IFMT) == S_IFIFO)
+#define S_ISFIFO(mode) (((mode) & S_IFMT) == S_IFIFO)
 #endif
 #if defined(S_IFSOCK) && defined(S_IFMT) && !defined(S_ISSOCK)
-#define S_ISSOCK(mode) ((mode & S_IFMT) == S_IFSOCK)
+#define S_ISSOCK(mode) (((mode) & S_IFMT) == S_IFSOCK)
 #endif
 #if defined(S_IFCHR) && defined(S_IFMT) && !defined(S_ISCHR)
-#define S_ISCHR(mode) ((mode & S_IFMT) == S_IFCHR)
+#define S_ISCHR(mode) (((mode) & S_IFMT) == S_IFCHR)
 #endif
 #if defined(S_IFDIR) && defined(S_IFMT) && !defined(S_ISDIR)
-#define S_ISDIR(mode) ((mode & S_IFMT) == S_IFDIR)
+#define S_ISDIR(mode) (((mode) & S_IFMT) == S_IFDIR)
+#endif
+
+/*
+ * SunOS 4.1.3: `man 2V open' has only one line that mentions O_NOBLOCK:
+ *
+ *     O_NONBLOCK     Same as O_NDELAY above.
+ *
+ * on the very same SunOS 4.1.3, I traced the open system call and found
+ * that an open("/dev/ttyy08", O_RDWR|O_NONBLOCK|O_NOCTTY) was blocked,
+ * whereas open("/dev/ttyy08", O_RDWR|O_NDELAY  |O_NOCTTY) went through.
+ *
+ * For this simple reason I now favour O_NDELAY. jw. 4.5.95
+ */
+#if defined(sun) && !defined(SVR4)
+# undef O_NONBLOCK
 #endif
 
 #if !defined(O_NONBLOCK) && defined(O_NDELAY)
